@@ -10,7 +10,6 @@ from pygitrepo.actions import Actions, actions
 from pygitrepo.repo_config import RepoConfig
 from pygitrepo.pkg.mini_colorma import Fore, Style
 
-
 SUB_COMMAND = "sub_command"
 
 
@@ -21,7 +20,6 @@ def colorful(message):
 parser = argparse.ArgumentParser(
     prog="pygitrepo",
 )
-
 subparser = parser.add_subparsers(
     title="sub commands",
     description="run automation script",
@@ -38,8 +36,14 @@ for method_name, method in Actions.__dict__.items():
             name=method._subcommand_name,
             help=Fore.CYAN + method._subcommand_help + Style.RESET_ALL
         )
+        subcommand_parser.add_argument(
+            "--do-dry-run",
+            action="store_true",
+            help="display info, doesn't take effect",
+        )
         action_mapper[method._subcommand_name] = getattr(actions, method_name)
 
+
 # manually add more important subc  ```````ommand parser
 class AddtionalSubCommandEnum:
     get_value = None
@@ -77,16 +81,23 @@ def main():  # pragma: no cover
     """
     Command Line Interface entry point.
     """
-    args = parser.parse_args()
+    # defined arguments stored in args, a named tuple object
+    # additional undefined arguments stored in unknown
+    args, unknown = parser.parse_known_args()
+    print(args)
     if args.sub_command is None:
         parser.parse_args(["-h"])
         return
-    
+
     repo_config = RepoConfig()
     repo_config.read_pygitrepo_config_file()
 
     if args.sub_command in action_mapper:
-        action_mapper[args.sub_command](repo_config)
+        action_mapper[args.sub_command](
+            repo_config,
+            _dry_run=args.do_dry_run,
+            _args=unknown,
+        )
     elif args.sub_command == AddtionalSubCommandEnum.get_value:
         get_value(repo_config, args.attr_name)
     else:
